@@ -5,11 +5,13 @@ use warnings;
 package MooX::Press;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.047';
+our $VERSION   = '0.049';
 
 use Types::Standard 1.008003 -is, -types;
 use Types::TypeTiny qw(ArrayLike HashLike);
 use Exporter::Tiny qw(mkopt);
+use Import::Into;
+use Module::Runtime qw(use_module);
 use namespace::autoclean;
 
 # Options not to carry up into subclasses;
@@ -75,16 +77,22 @@ sub _apply_default_options {
 		}
 	}
 	
+	my $no_warn = exists($opts->{factory_package});
+	
 	$opts->{factory_package} = defined($opts->{prefix}) ? $opts->{prefix} : 'Local'
 		unless exists $opts->{factory_package};
 	
-	if (defined $opts->{factory_package} and $opts->{factory_package} eq 'Local') {
+	if (!$no_warn and defined($opts->{factory_package}) and $opts->{factory_package} eq 'Local') {
 		require FindBin;
 		if ($FindBin::Script ne '-e') {
 			require Carp;
 			Carp::carp('Using "Local" as factory; please set prefix or factory_package');
 		}
 	}
+	
+	unless (exists $opts->{type_library}) {
+		$opts->{type_library} = $builder->qualify_name('Types', $opts->{prefix});
+	}
 }
 
 sub import {
@@ -132,11 +140,6 @@ sub import {
 		$builder->munge_class_options($pkg->[1], \%opts);
 	}
 
-	unless (exists $opts{type_library}) {
-		$opts{type_library} = 'Types';
-		$opts{type_library} = $builder->qualify_name($opts{type_library}, $opts{prefix});
-	}
-	
 	if ($opts{type_library}) {
 		$builder->prepare_type_library($opts{type_library}, %opts);
 		# no type for role generators
@@ -242,14 +245,14 @@ sub munge_role_generator_options {
 }
 
 sub qualify_name {
-	shift;
+	my $me = shift;
 	my ($name, $prefix, $parent) = @_;
 	my $sigil = "";
 	if ($name =~ /^[@%\$]/) {
 		$sigil = substr $name, 0, 1;
 		$name  = substr $name, 1;
 	}
-	return $sigil.join("::", $parent->$_handle_list, $1) if (defined $parent and $name =~ /^\+(.+)/);
+	$name = join("::", '', $parent->$_handle_list, $1) if (defined $parent and $name =~ /^\+(.+)/);
 	return $sigil.$1 if $name =~ /^::(.+)$/;
 	$prefix ? $sigil.join("::", $prefix, $name) : $sigil.$name;
 }
@@ -281,9 +284,7 @@ sub prepare_type_library {
 	require Type::Tiny::Role;
 	require Type::Tiny::Class;
 	require Type::Registry;
-	eval "package $lib; use Type::Library -base; 1"
-		or $builder->croak("Could not prepare type library $lib: $@");
-	require Module::Runtime;
+	use_module('Type::Library')->import::into($lib, -base);
 	$INC{ Module::Runtime::module_notional_filename($lib) } = __FILE__;
 	my $adder = sub {
 		my $me = shift;
@@ -550,6 +551,8 @@ sub _make_package {
 	my $qname = $builder->qualify_name($name, $opts{prefix}, @isa);
 	my $tn = $builder->type_name($qname, $opts{prefix});
 	
+	print "$name -> $qname\n";
+	
 	if (!exists $opts{factory}) {
 		$opts{factory} = $opts{abstract} ? undef : sprintf('new_%s', lc $tn);
 	}
@@ -563,21 +566,16 @@ sub _make_package {
 	if ($opts{is_role}) {
 		no strict 'refs';
 		return if ${"$qname\::BUILT"};
-		
-		eval "package $qname; use $toolkit\::Role; use namespace::autoclean; our \$BUILT = 1"
-			or $builder->croak("Could not create package $qname: $@");
+		use_module("$toolkit\::Role")->import::into($qname);
+		use_module("namespace::autoclean")->import::into($qname);
+		${"$qname\::BUILT"} = 1;
 	}
 	else {
-		my $optthing = '';
-		if ($toolkit eq 'Moo') {
-			$optthing = ' use MooX::TypeTiny;'      if eval { require MooX::TypeTiny };
-		}
-		elsif ($toolkit eq 'Moose') {
-			$optthing = ' use MooseX::XSAccessor;'  if eval { require MooseX::XSAccessor };
-		}
-		eval "package $qname; use $toolkit;$optthing use namespace::autoclean; 1"
-			or $builder->croak("Could not create package $qname: $@");
-	
+		use_module($toolkit)->import::into($qname);
+		use_module("MooX::TypeTiny")->import::into($qname) if $toolkit eq 'Moo' && eval { require MooX::TypeTiny };
+		use_module("MooseX::XSAccessor")->import::into($qname) if $toolkit eq 'Moose' && eval { require MooseX::XSAccessor };
+		use_module("namespace::autoclean")->import::into($qname);
+
 		my $method  = $opts{toolkit_extend_class} || ("extend_class_".lc $toolkit);
 		if (@isa) {
 			$builder->$method($qname, \@isa);
@@ -612,9 +610,7 @@ sub _make_package {
 			elsif (is_ArrayRef($imports[0])) {
 				@params = @{ shift @imports };
 			}
-			require Import::Into;
-			eval "require $import";
-			$import->import::into($qname, @params);
+			use_module($import)->import::into($qname, @params);
 		}
 	}
 	
@@ -774,7 +770,7 @@ sub _make_package {
 				my ($shv_toolkit, $shv_data);
 				if ($spec{handles_via}) {
 					$shv_toolkit = "Sub::HandlesVia::Toolkit::$toolkit";
-					eval "require $shv_toolkit" or die($@);
+					use_module($shv_toolkit);
 					$shv_data = $shv_toolkit->clean_spec($qname, $attrname, \%spec);
 				}
 				$builder->$method($qname, $attrname, \%spec);
